<?php
/**
 * Gutenberg blocks and templates.
 *
 * @link       https://www.boldermail.com/about/
 * @since      2.0.0
 *
 * @package    Boldermail
 * @subpackage Boldermail/admin
 * @author     Hernan Villanueva <hernan@boldermail.com>
 */

defined( 'ABSPATH' ) || exit;

/**
 * Boldermail_Gutenberg class.
 *
 * @since 2.0.0
 */
class Boldermail_Gutenberg {

	/**
	 * Initialize the hooks.
	 *
	 * @since  2.0.0
	 * @return void
	 */
	public static function init() {

		add_filter( 'block_categories', array( __CLASS__, 'block_categories' ), 10, 2 );
		add_action( 'init', array( __CLASS__, 'register_post_meta' ), PHP_INT_MAX );
		add_action( 'init', array( __CLASS__, 'editor_styles' ), PHP_INT_MAX );
		add_filter( 'allowed_block_types', array( __CLASS__, 'allowed_block_types' ), PHP_INT_MAX, 2 );
		add_action( 'init', array( __CLASS__, 'allowed_block_patterns' ), PHP_INT_MAX, 2 );
		add_filter( 'block_editor_settings', array( __CLASS__, 'block_editor_settings' ), PHP_INT_MAX, 2 );
		add_filter( 'use_block_editor_for_post', array( __CLASS__, 'use_block_editor_for_post' ), PHP_INT_MAX, 2 );
		add_filter( 'use_block_editor_for_post_type', array( __CLASS__, 'use_block_editor_for_post_type' ), PHP_INT_MAX, 2 );
		add_filter( 'gutenberg_can_edit_post_type', array( __CLASS__, 'use_block_editor_for_post_type' ), 10, 2 );
		add_filter( 'image_size_names_choose', array( __CLASS__, 'image_size_names_choose' ), PHP_INT_MAX );

	}

	/**
	 * Filter the settings to pass to the block editor.
	 *
	 * @see    https://stackoverflow.com/a/58783057/1991500
	 * @since  2.0.0
	 * @param  array   $settings Default editor settings.
	 * @param  WP_Post $post     Post object.
	 * @return array             Modified editor settings.
	 */
	public static function block_editor_settings( $settings, $post ) {

		if ( ! in_array( $post->post_type, [ 'bm_block_template', 'bm_template' ], true ) ) {
			return $settings;
		}

		// Remove default styles.
		unset( $settings['styles'][0] );

		return $settings;

	}

	/**
	 * Remove editor styles enqueued by the main theme, and then enqueue our
	 * block assets.
	 *
	 * @since  2.0.0
	 * @return void
	 */
	public static function editor_styles() {

		if ( in_array( boldermail_get_current_screen_post_type(), [ 'bm_block_template', 'bm_template' ], true ) ) {

			// Clear all styles and customizations made by other plugins or themes.
			remove_editor_styles();
			remove_all_actions( 'enqueue_block_editor_assets' );
			remove_theme_support( 'editor-color-palette' );
			remove_theme_support( 'editor-font-sizes' );

			// Add our own customizations.
			add_action( 'enqueue_block_editor_assets', array( __CLASS__, 'enqueue_block_editor_assets' ), PHP_INT_MAX );

		}

	}

	/**
	 * Enqueue block assets for the backend editor.
	 *
	 * @since  2.0.0
	 * @return void
	 */
	public static function enqueue_block_editor_assets() {

		if ( ! in_array( get_post_type(), [ 'bm_block_template', 'bm_template' ], true ) ) {
			return;
		}

		$suffix = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ? '' : '.min';

		$path = 'includes/gutenberg/dist/blocks.build.js';
		wp_enqueue_script( 'boldermail-gutenberg-blocks', BOLDERMAIL_PLUGIN_URL . $path, array( 'boldermail-editor', 'wp-i18n', 'wp-element', 'wp-blocks', 'wp-components', 'wp-editor', 'wp-plugins', 'wp-edit-post' ), filemtime( BOLDERMAIL_PLUGIN_DIR . $path ), false );

		$path = 'includes/gutenberg/dist/blocks.editor.build.css';
		wp_enqueue_style( 'boldermail-editor-css', BOLDERMAIL_PLUGIN_URL . $path, array( 'wp-edit-blocks' ), filemtime( BOLDERMAIL_PLUGIN_DIR . $path ) );

		$path = "assets/css/boldermail-editor-styles-gutenberg$suffix.css";
		wp_enqueue_style( 'boldermail-editor-styles-gutenberg-css', BOLDERMAIL_PLUGIN_URL . $path, array( 'wp-edit-blocks' ), filemtime( BOLDERMAIL_PLUGIN_DIR . $path ) );

	}

	/**
	 * Register all Boldermail post meta for Gutenberg.
	 *
	 * @since  2.0.0
	 * @return void
	 */
	public static function register_post_meta() {

		if ( ! is_blog_installed() || ! post_type_exists( 'bm_block_template' ) || ! post_type_exists( 'bm_template' ) ) {
			return;
		}

		do_action( 'boldermail_before_register_post_meta' );

		$block_editor_meta_args = apply_filters(
			'bm_block_editor_post_meta_args',
			array(
				'show_in_rest'  => true,
				'single'        => true,
				'type'          => 'string',
				'auth_callback' => function() {
					return current_user_can( 'edit_posts' );
				},
			)
		);

		register_post_meta( 'bm_block_template', '_template_style', $block_editor_meta_args );
		register_post_meta( 'bm_template', '_template_style', $block_editor_meta_args );

		do_action( 'boldermail_after_register_post_meta' );

	}

	/**
	 * Add block categories for the editor.
	 *
	 * @see    https://loomo.ca/gutenberg-creating-custom-block-categories/
	 * @since  2.0.0
	 * @param  array   $categories Array of block categories.
	 * @param  WP_Post $post       Post object.
	 * @return array
	 */
	public static function block_categories( $categories, $post ) {

		if ( ! in_array( $post->post_type, [ 'bm_block_template', 'bm_template' ], true ) ) {
			return $categories;
		}

		return array_merge(
			$categories,
			array(
				array(
					'slug'  => 'boldermail-text',
					'title' => __( 'Text', 'boldermail' ),
				),
			),
			array(
				array(
					'slug'  => 'boldermail-media',
					'title' => __( 'Media', 'boldermail' ),
				),
			),
			array(
				array(
					'slug'  => 'boldermail-design',
					'title' => __( 'Design', 'boldermail' ),
				),
			)
		);

	}

	/**
	 * Filter the allowed block types for the editor.
	 *
	 * @since  2.0.0
	 * @param  bool|array $allowed_blocks Array of block type slugs, or boolean to enable/disable all.
	 * @param  WP_Post    $post           The post resource data.
	 * @return bool|array
	 */
	public static function allowed_block_types( $allowed_blocks, $post ) {

		/**
		 * When the template is published or updated, it copies its
		 * content to a Reusable Block post. If the user then insert
		 * this Reusable Block inside the template, it creates a
		 * recursion loop, and it causes an error. This is reported
		 * as an issue in the Gutenberg project.
		 *
		 * We do not include these Boldermail blocks in `wp_block`,
		 * because then we would have to enqueue our scripts, which
		 * modify the UI for other posts. The user should use the template
		 * post as the UI instead.
		 *
		 * @see   https://github.com/WordPress/gutenberg/issues/22131
		 * @since 2.2.0
		 */
		if ( in_array( $post->post_type, [ 'bm_block_template', 'bm_template' ], true ) ) {
			return [
				'boldermail/template',
				'boldermail/paragraph',
				'boldermail/heading',
				'boldermail/list',
				'boldermail/image',
				'boldermail/button',
				'boldermail/boxed-text',
				'boldermail/separator',
				'boldermail/social-links',
				'boldermail/social-link',
				'boldermail/social-shares',
				'boldermail/social-share',
				'boldermail/columns',
				'boldermail/column',
				'boldermail/referral',
				'core/block', // Reusable block.
			];
		}

		return $allowed_blocks;

	}

	/**
	 * Disable the WordPress default block patterns.
	 *
	 * @see   https://github.com/WordPress/gutenberg/issues/24505
	 * @since 2.2.0
	 */
	public static function allowed_block_patterns() {

		if ( in_array( boldermail_get_current_screen_post_type(), [ 'bm_block_template', 'bm_template' ], true ) ) {

			if ( ! class_exists( 'WP_Block_Patterns_Registry' ) ) {
				return;
			}

			$default_block_patterns = WP_Block_Patterns_Registry::get_instance()->get_all_registered();

			foreach ( $default_block_patterns as $block_pattern ) {
				unregister_block_pattern( $block_pattern['name'] );
			}

		}

	}

	/**
	 * Filter whether a post is able to be edited in the block editor.
	 *
	 * Activate the Gutenberg editor for the block templates and templates.
	 * This filter is used in the Classic Editor plugin if the administrator allows
	 * users to select the Classic or Block Editor.
	 *
	 * @since  2.0.0
	 * @param  bool    $use_block_editor Whether the post can be edited or not.
	 * @param  WP_Post $post             The post being checked.
	 * @return bool
	 */
	public static function use_block_editor_for_post( $use_block_editor, $post ) {

		if ( 'bm_template' === $post->post_type ) {

			$template = boldermail_get_template( $post );

			if ( ! $template ) {
				return $use_block_editor;
			}

			if ( $template->use_block_editor() ) {
				return true;
			} else {
				remove_post_type_support( 'bm_template', 'editor' );
				return false;
			}

		}

		if ( 'bm_block_template' === $post->post_type ) {
			return true;
		}

		return $use_block_editor;

	}

	/**
	 * Filter whether a post type is able to be edited in the block editor.
	 *
	 * Activate the Gutenberg editor for the block templates, and deactivate for newsletters.
	 * This filter is used in the Classic Editor plugin if the administrator does
	 * not allow users to select an editor.
	 *
	 * @since  2.0.0
	 * @param  bool   $use_block_editor Whether the post type can be edited or not. Default true.
	 * @param  string $post_type        The post type being checked.
	 * @return bool
	 */
	public static function use_block_editor_for_post_type( $use_block_editor, $post_type ) {

		if ( in_array( $post_type, [ 'bm_newsletter', 'bm_newsletter_ares', 'bm_template' ], true ) ) {
			return false;
		}

		if ( 'bm_block_template' === $post_type ) {
			return true;
		}

		return $use_block_editor;

	}

	/**
	 * Filter the names and labels of the default image sizes in the block editor.
	 * Even though the Image block uses its own setting `bmImageSizes` to select
	 * which image sizes to show in the Inspector Controls, we still need to include
	 * the slug in this function which gets called from `wp_prepare_attachment_for_js`.
	 *
	 * @since  2.0.0
	 * @param  string[] $sizes Array of image size labels keyed by their name.
	 * @return string[]
	 */
	public static function image_size_names_choose( $sizes ) {

		return array_merge(
			$sizes,
			array(
				'boldermail_newsletter' => __( 'Boldermail Newsletter', 'boldermail' ),
			)
		);

	}

}

Boldermail_Gutenberg::init();
